# -*- coding: utf-8 -*-

import os
import sys
from ctypes import CDLL


# Exception strings used when checking inputs
_neg_err = "{name} must be >= 0.0"
_hour_range_err = "{name} must be in the range 0.0 <= {name} < 24.0"
_elev_range_err = "{name} must be in the range -90.0 <= {name} <= 90.0"
_az180_range_err = "{name} must be in the range -180.0 <= {name} <= 180.0"
_az360_range_err = "{name} must be in the range 0.0 <= {name} <= 360.0"
_vector_len_err = "{name} must be a sequence of length 3"
_option_err = "{name} must be in {allowed}"
_jd_year_err = "{name} must be >= -4712"
_month_range_err = "{name} must be 1 <= {name} <= 12"
_day_range_err = "{name} must be 1 <= {name} <= {daysinmonth}"

_days_in_month = (None, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)


class NonConvergentError(Exception):
    pass


class IndeterminateError(Exception):
    pass


class InitializationError(Exception):
    pass


def _check_c_errors(retval, func, args):
    """
    Function to check return values from wrapped C functions.

    This function is used in conjunction with the ctypes ``errcheck`` attribute
    attached to a C function object. Assigning this function to the
    ``errcheck`` attribute causes an automatic check of the return value from
    the C function. Every C function object that utilizes this ``errcheck``
    feature must also have an attribute called ``c_errors`` assigned to it.
    ``c_errors`` is a dictionary where the keys are possible return values and
    the values are length-2 tuples; each tuple consists of the exception to be
    raised and the message sent to that exception.

    https://docs.python.org/3.4/library/ctypes.html#ctypes._FuncPtr.errcheck

    """
    if retval:
        error = func.c_errors[retval]
        raise error[0](error[1])


def get_and_configure_library():
    for directory in sys.path:
        path_to_libnovas = os.path.join(directory, 'novas', 'libnovas.so')
        if os.path.isfile(path_to_libnovas):
            libnovas = CDLL(path_to_libnovas)

    return libnovas

novaslib = get_and_configure_library()



# -*- coding: utf-8 -*-

import ctypes
from novas import novaslib, _check_c_errors
from novas import _neg_err, _option_err


def solarsystem(tjd, body, origin):
    """
    Provides an interface between the JPL direct-access solar system
    ephemerides and NOVAS-C.

    Parameters
    ----------
    tjd : float
        Julian date of the desired time, on the TDB time scale.
    body : integer
        Body identification number for the solar system object of
        interest; Mercury = 1, ..., Pluto = 9, Sun = 10, Moon = 11.
    origin : integer
        Origin code
            = 0 ... solar system barycenter
            = 1 ... center of mass of the Sun
            = 2 ... center of Earth

    Returns
    -------
    position : tuple of floats, of length 3
        Position vector of 'body' at 'tjd'; equatorial rectangular
        coordinates in AU referred to the ICRS.
    velocity : tuple of floats, of length 3
        Velocity vector of 'body' at 'tjd'; equatorial rectangular
        system referred to the ICRS, in AU/day.

    Notes
    -----
    .. [N1] This function and function 'planet_ephemeris' were
        designed to work with the 1997 version of the JPL
        ephemerides, as noted in the references.
    .. [N2] The user must have a JPL binary ephemeris file (see
        package README) and open the file using function
        'ephem_open' prior to calling this function.
    .. [N3] This function places the entire Julian date in the first
        element of the input time to 'planet_ephemeris'. This is
        adequate for all but the highest precision applications. For
        highest precision, use function 'solarsystem_hp'.

    References
    ----------
    .. [R1] JPL. 2007, "JPL Planetary and Lunar Ephemerides: Export
        Information," (Pasadena, CA: JPL)
        http://ssd.jpl.nasa.gov/?planet_eph_export.
    .. [R2] Kaplan, G. H. "NOVAS: Naval Observatory Vector
        Astrometry Subroutines"; USNO internal document dated 20 Oct
        1988; revised 15 Mar 1990.

    """

    if tjd < 0.0:
        raise ValueError(_neg_err.format(name='tjd'))
    if body not in list(range(1, 12)):
        raise ValueError(_option_err.format(name='body',
                                            allowed=list(range(1, 12))))
    if origin not in [0, 1, 2]:
        raise ValueError(_option_err.format(name='origin',
                                            allowed=[0, 1, 2]))

    _solarsystem = novaslib.solarsystem
    _solarsystem.argtypes = (ctypes.c_double, ctypes.c_short, ctypes.c_short,
                             ctypes.POINTER(ctypes.c_double*3),
                             ctypes.POINTER(ctypes.c_double*3))
    _solarsystem.restype = ctypes.c_short
    _solarsystem.errcheck = _check_c_errors
    _solarsystem.c_errors = {
        1: (ValueError, "from C function 'solarsystem': Invalid value of body or origin."),
        2: (RuntimeError, "from C function 'solarsystem': Error detected by JPL software.")
        }

    position = (ctypes.c_double*3)()
    velocity = (ctypes.c_double*3)()

    _solarsystem(tjd, body, origin, ctypes.byref(position),
                 ctypes.byref(velocity))

    return tuple([i for i in position]), tuple([j for j in velocity])


def solarsystem_hp(tjd, body, origin):
    """
    Provides an interface between the JPL direct-access solar system
    ephemerides and NOVAS-C for highest precision applications.

    Parameters
    ----------
    tjd : tuple of floats, of length 2
        Two-element tuple containing the Julian date, which may be
        split any way (although the first element is usually the
        "integer" part, and the second element is the "fractional"
        part). Julian date is on the TDB or "T_eph" time scale.
    body : integer
        Body identification number for the solar system object of
        interest; Mercury = 1, ..., Pluto = 9, Sun = 10, Moon = 11.
    origin : integer
        Origin code
            = 0 ... solar system barycenter
            = 1 ... center of mass of the Sun
            = 2 ... center of Earth

    Returns
    -------
    position : tuple of floats, of length 3
        Position vector of 'body' at 'tjd'; equatorial rectangular
        coordinates in AU referred to the ICRS.
    velocity : tuple of floats, of length 3
        Velocity vector of 'body' at 'tjd'; equatorial rectangular
        system referred to the ICRS, in AU/day.

    Notes
    -----
    .. [N1] This function and function 'planet_ephemeris' were
        designed to work with the 1997 version of the JPL
        ephemerides, as noted in the references.
    .. [N2] The user must have a JPL binary ephemeris file (see
        package README) and open the file using function
        'ephem_open' prior to calling this function.
    .. [N3] This function supports the "split" Julian date feature
        of function 'planet_ephemeris' for highest precision. For
        usual applications, use function 'solarsystem'.

    References
    ----------
    .. [R1] JPL. 2007, "JPL Planetary and Lunar Ephemerides: Export
        Information," (Pasadena, CA: JPL)
        http://ssd.jpl.nasa.gov/?planet_eph_export.
    .. [R2] Kaplan, G. H. "NOVAS: Naval Observatory Vector
        Astrometry Subroutines"; USNO internal document dated 20 Oct
        1988; revised 15 Mar 1990.

    """

    if tjd < 0.0:
        raise ValueError(_neg_err.format(name='tjd'))
    if body not in list(range(1, 12)):
        raise ValueError(_option_err.format(name='body',
                                            allowed=list(range(1, 12))))
    if origin not in [0, 1, 2]:
        raise ValueError(_option_err.format(name='origin',
                                            allowed=[0, 1, 2]))

    _solarsystem_hp = novaslib.solarsystem_hp
    _solarsystem_hp.argtypes = (ctypes.c_double*2, ctypes.c_short,
                                ctypes.c_short,
                                ctypes.POINTER(ctypes.c_double*3),
                                ctypes.POINTER(ctypes.c_double*3))
    _solarsystem_hp.restype = ctypes.c_short
    _solarsystem_hp.errcheck = _check_c_errors
    _solarsystem_hp.c_errors = {
        1: (ValueError, "from C function 'solarsystem_hp': Invalid value of body or origin."),
        2: (RuntimeError, "from C function 'solarsystem_hp': Error detected by JPL software.")
        }

    position = (ctypes.c_double*3)()
    velocity = (ctypes.c_double*3)()

    _solarsystem_hp((ctypes.c_double*2)(*tjd), body, origin,
                    ctypes.byref(position), ctypes.byref(velocity))

    return tuple([i for i in position]), tuple([j for j in velocity])
